@@ -8,7 +8,7 @@ Perl::Dist::WiX::Installation - Basic installation routines
 
 =head1 VERSION
 
-This document describes Perl::Dist::WiX::Installation version 1.102001.
+This document describes Perl::Dist::WiX::Installation version 1.200.
 
 =head1 DESCRIPTION
 
@@ -18,6 +18,7 @@ install files.
 =head1 SYNOPSIS
 
 	# This module is not to be used independently.
+	# It provides methods to be called on a Perl::Dist::WiX object.
 
 =head1 INTERFACE
 
@@ -38,7 +39,7 @@ require Perl::Dist::WiX::Asset::Module;
 require Perl::Dist::WiX::Asset::PAR;
 require Perl::Dist::WiX::Asset::Website;
 
-our $VERSION = '1.102001';
+our $VERSION = '1.200';
 $VERSION =~ s/_//ms;
 
 =pod
@@ -57,6 +58,8 @@ Returns true or throws an exception on error.
 
 =cut
 
+
+
 sub install_binary {
 	my $self   = shift;
 	my $binary = Perl::Dist::WiX::Asset::Binary->new(
@@ -70,6 +73,8 @@ sub install_binary {
 	return $filelist;
 } ## end sub install_binary
 
+
+
 =head2 install_library
 
   $self->install_library(
@@ -84,6 +89,8 @@ Returns true or throws an exception on error.
 
 =cut
 
+
+
 sub install_library {
 	my $self    = shift;
 	my $library = Perl::Dist::WiX::Asset::Library->new(
@@ -97,6 +104,7 @@ sub install_library {
 }
 
 
+
 =pod
 
 =head2 install_distribution
@@ -144,6 +152,8 @@ Returns true or throws an exception on error.
 
 =cut
 
+
+
 sub install_distribution {
 	my $self = shift;
 	my $dist = Perl::Dist::WiX::Asset::Distribution->new(
@@ -156,6 +166,8 @@ sub install_distribution {
 	return $self;
 }
 
+
+
 =pod
 
 =head2 install_distribution_from_file
@@ -199,6 +211,8 @@ Returns true or throws an exception on error.
 
 =cut
 
+
+
 sub install_distribution_from_file {
 	my $self = shift;
 
@@ -219,8 +233,7 @@ sub install_distribution_from_file {
 	return $self;
 } ## end sub install_distribution_from_file
 
-## search_packlists should not be needed.
-## If it's needed, pull it out of version 1.000.
+
 
 =pod
 
@@ -249,10 +262,15 @@ the module.
 The optional 'packlist' param sshould be 0 if a .packlist file is not 
 installed with the module.
 
+This does NOT install the dependencies of the module named - they have
+to be installed before the named module, in one way or another.
+
 Returns true or throws an exception on error.
 
 =cut
 
+
+
 sub install_module {
 	my $self   = shift;
 	my $module = Perl::Dist::WiX::Asset::Module->new(
@@ -273,6 +291,8 @@ sub install_module {
 	return $self;
 } ## end sub install_module
 
+
+
 =pod
 
 =head2 install_modules
@@ -291,6 +311,8 @@ underlying C<install_module> call other than the name.
 
 =cut
 
+
+
 sub install_modules {
 	my $self = shift;
 
@@ -301,6 +323,8 @@ sub install_modules {
 	return $self;
 }
 
+
+
 =pod
 
 =head2 install_par
@@ -317,6 +341,8 @@ Returns true on success or throws an exception on error.
 
 =cut
 
+
+
 sub install_par {
 	my $self = shift;
 
@@ -340,6 +366,8 @@ sub install_par {
 	return $self;
 } ## end sub install_par
 
+
+
 =pod
 
 =head2 install_file
@@ -394,6 +422,8 @@ or throws an exception on error.
 
 =cut
 
+
+
 sub install_file {
 	my $self = shift;
 	my $file = Perl::Dist::WiX::Asset::File->new(
@@ -406,6 +436,8 @@ sub install_file {
 	return $filelist;
 }
 
+
+
 =pod
 
 =head2 install_launcher
@@ -433,6 +465,8 @@ Returns true or throws an exception on error.
 
 =cut
 
+
+
 sub install_launcher {
 	my $self     = shift;
 	my $launcher = Perl::Dist::WiX::Asset::Launcher->new(
@@ -445,6 +479,8 @@ sub install_launcher {
 	return $self;
 }
 
+
+
 =pod
 
 =head2 install_website
@@ -475,6 +511,8 @@ Returns true on success, or throws an exception on error.
 
 =cut
 
+
+
 sub install_website {
 	my $self    = shift;
 	my $website = Perl::Dist::WiX::Asset::Website->new(
@@ -515,7 +553,7 @@ Curtis Jewell E<lt>csjewell@cpan.orgE<gt>
 
 =head1 SEE ALSO
 
-L<Perl::Dist|Perl::Dist>, L<Perl::Dist::WiX|Perl::Dist::WiX>, 
+L<Perl::Dist::WiX|Perl::Dist::WiX>, 
 L<http://ali.as/>, L<http://csjewell.comyr.com/perl/>
 
 =head1 COPYRIGHT AND LICENSE
