@@ -8,7 +8,7 @@ Perl::Dist::WiX::Libraries - Library installation routines
 
 =head1 VERSION
 
-This document describes Perl::Dist::WiX::Libraries version 1.102.
+This document describes Perl::Dist::WiX::Libraries version 1.200.
 
 =head1 DESCRIPTION
 
@@ -18,6 +18,7 @@ install the C toolchain and library files.
 =head1 SYNOPSIS
 
 	# This module is not to be used independently.
+	# It provides methods to be called on a Perl::Dist::WiX object.
 
 =head1 INTERFACE
 
@@ -31,7 +32,7 @@ use Params::Util qw( _STRING );
 use Perl::Dist::WiX::Exceptions;
 use Readonly;
 
-our $VERSION = '1.102';
+our $VERSION = '1.200';
 $VERSION =~ s/_//ms;
 
 Readonly my %PACKAGES => (
@@ -40,6 +41,7 @@ Readonly my %PACKAGES => (
 		'mingw-make'    => 'mingw32-make-3.81-2.tar.gz',
 		'pexports'      => '32bit-gcc3/pexports-0.43-1-20100120.zip',
 		'gcc-toolchain' => 'mingw32-gcc3-toolchain-20091026-subset.tar.gz',
+		'gcc-license'   => undef,
 
 # Former components of what's now included in gcc-toolchain.
 #		'gcc-core'      => 'gcc-core-3.4.5-20060117-3.tar.gz',
@@ -49,16 +51,18 @@ Readonly my %PACKAGES => (
 #		'w32api'        => 'w32api-3.10.tar.gz',
 	},
 	'32bit-gcc4' => {
-		'dmake'      => '32bit-gcc4/dmake-SVN20091127-bin_20100120.zip',
-		'mingw-make' => '32bit-gcc4/gmake-3.81-20090914-bin_20010120.zip',
-		'pexports'   => '32bit-gcc4/pexports-0.43-1.zip',
-		'gcc-toolchain' => '32bit-gcc4/mingw-w32-20091019_subset.7z',
+		'dmake'      => '32bit-gcc4/dmake-SVN20091127-bin_20100308.zip',
+		'mingw-make' => '32bit-gcc4/gmake-3.81-20090914-bin_20100120.zip',
+		'pexports'   => '32bit-gcc4/pexports-0.44-bin_20100120.zip',
+		'gcc-toolchain' => '32bit-gcc4/mingw64-w32-20100123-kmx-v2.zip',
+		'gcc-license'   => '32bit-gcc4/mingw64-w32-20100123-kmx-v2-lic.zip',
 	},
 	'64bit-gcc4' => {
-		'dmake'         => 'dmake-4.8-20070327-SHAY.zip',
-		'mingw-make'    => 'mingw32-make-3.81-2.tar.gz',
-		'pexports'      => 'pexports-0.43-1.zip',
-		'gcc-toolchain' => 'mingw-w32-20091019_subset.7z',
+		'dmake'         => '64bit-gcc4/dmake-SVN20091127-bin_20100308.zip',
+		'mingw-make'    => '64bit-gcc4/gmake-3.81.90_20100127_20100305.zip',
+		'pexports'      => '64bit-gcc4/pexports-0.44-bin_20100110.zip',
+		'gcc-toolchain' => '64bit-gcc4/mingw64-w64-20100123-kmx-v2.zip',
+		'gcc-license'   => '64bit-gcc4/mingw64-w64-20100123-kmx-v2-lic.zip',
 	},
 );
 
@@ -70,10 +74,12 @@ Readonly my %PACKAGES => (
 
 The C<library_directory> method returns the correct directory on the
 strawberryperl.com server for libraries, given the L<bits()|Perl::Dist::WiX/bits> 
-and L<gcc-version()|Perl::Dist::WiX/gcc_version> values.
+and L<gcc_version()|Perl::Dist::WiX/gcc_version> values.
 
 =cut
 
+
+
 sub library_directory {
 	my $self = shift;
 
@@ -82,6 +88,9 @@ sub library_directory {
 	return $answer;
 }
 
+
+
+# Private routine to get the filename for a package.
 sub _binary_file {
 	my $self    = shift;
 	my $package = shift;
@@ -106,6 +115,8 @@ sub _binary_file {
 	return $package_file;
 } ## end sub _binary_file
 
+
+# Private troutine to map a file or package name to a URL.
 sub _binary_url {
 	my $self = shift;
 	my $file = shift;
@@ -114,7 +125,7 @@ sub _binary_url {
 	unless ( _STRING($file) ) {
 		PDWiX::Parameter->throw(
 			parameter => 'file',
-			where     => '->binary_url'
+			where     => '->_binary_url'
 		);
 	}
 
@@ -123,7 +134,7 @@ sub _binary_url {
 		# Shorthand, map to full file name
 		$file = $self->_binary_file( $file, @_ );
 	}
-	return $self->binary_root . q{/} . $file;
+	return $self->binary_root() . q{/} . $file;
 } ## end sub _binary_url
 
 
@@ -137,7 +148,7 @@ sub _binary_url {
 
   $dist->install_gcc_toolchain()
 
-The C<install_dmake> method installs the corrent gcc toolchain into the
+The C<install_gcc_toolchain> method installs the corrent gcc toolchain into the
 distribution, and is typically installed during "C toolchain" build
 phase.
 
@@ -148,35 +159,54 @@ Returns true or throws an exception on error.
 
 =cut
 
+
+
 sub install_gcc_toolchain {
 	my $self = shift;
 
 	# Install the gcc toolchain
 	my $filelist = $self->install_binary(
-		name    => 'gcc-toolchain',
-		url     => $self->_binary_url('gcc-toolchain'),
-		license => {
-			'COPYING'     => 'gcc/COPYING',
-			'COPYING.lib' => 'gcc/COPYING.lib',
-		},
+		name => 'gcc-toolchain',
+		url  => $self->_binary_url('gcc-toolchain'),
+		( 32 == $self->bits() )
+		? (
+			license => {
+				'COPYING'     => 'gcc/COPYING',
+				'COPYING.lib' => 'gcc/COPYING.lib',
+			},
+		  )
+		: (),
 	);
 
-	$self->insert_fragment( 'gcc_toolchain', $filelist );
+	my $overwritable = ( 3 == $self->gcc_version() ) ? 1 : 0;
+	$self->insert_fragment( 'gcc_toolchain', $filelist, $overwritable );
 
-	$self->_set_bin_dlltool( $self->_file( 'c', 'bin', 'dlltool.exe' ) );
+	# Initialize the dlltool location.
+	$self->_set_bin_dlltool( $self->file( 'c', 'bin', 'dlltool.exe' ) );
 	unless ( -x $self->bin_dlltool() ) {
 		PDWiX->throw(q{Can't execute dlltool});
 	}
 
+	# Install the licenses (they're in a different file for gcc4)
+	if ( 4 == $self->gcc_version() ) {
+		my $filelist2 = $self->install_binary(
+			name       => 'gcc-license',
+			url        => $self->_binary_url('gcc-license'),
+			install_to => q{.},
+		);
+		$self->insert_fragment( 'gcc_license', $filelist2 );
+	}
 
 	return 1;
 } ## end sub install_gcc_toolchain
 
+
+
 =pod
 
 =head2 install_dmake
 
-  $dist->install_dmake
+  $dist->install_dmake()
 
 The C<install_dmake> method installs the B<dmake> make tool into the
 distribution, and is typically installed during "C toolchain" build
@@ -189,6 +219,8 @@ Returns true or throws an exception on error.
 
 =cut
 
+
+
 sub install_dmake {
 	my $self = shift;
 
@@ -207,7 +239,7 @@ sub install_dmake {
 	);
 
 	# Initialize the make location
-	$self->_set_bin_make( $self->_file( 'c', 'bin', 'dmake.exe' ) );
+	$self->_set_bin_make( $self->file( 'c', 'bin', 'dmake.exe' ) );
 	unless ( -x $self->bin_make() ) {
 		PDWiX->throw(q{Can't execute make});
 	}
@@ -217,11 +249,13 @@ sub install_dmake {
 	return 1;
 } ## end sub install_dmake
 
+
+
 =pod
 
 =head2 install_pexports
 
-  $dist->install_pexports
+  $dist->install_pexports()
 
 The C<install_pexports> method installs the C<MinGW pexports> package
 into the distribution.
@@ -233,6 +267,8 @@ Returns true or throws an exception on error.
 
 =cut
 
+
+
 sub install_pexports {
 	my $self = shift;
 
@@ -241,7 +277,9 @@ sub install_pexports {
 		url        => $self->_binary_url('pexports'),
 		install_to => q{.},
 	);
-	$self->_set_bin_pexports( $self->_file( 'c', 'bin', 'pexports.exe' ) );
+
+	# Initialize the pexports location.
+	$self->_set_bin_pexports( $self->file( 'c', 'bin', 'pexports.exe' ) );
 	unless ( -x $self->bin_pexports() ) {
 		PDWiX->throw(q{Can't execute pexports});
 	}
@@ -251,11 +289,13 @@ sub install_pexports {
 	return 1;
 } ## end sub install_pexports
 
+
+
 =pod
 
 =head2 install_mingw_make
 
-  $dist->install_mingw_make
+  $dist->install_mingw_make()
 
 The C<install_mingw_make> method installs the MinGW build of the B<GNU make>
 build tool.
@@ -268,13 +308,35 @@ Returns true or throws an exception on error.
 
 =cut
 
+
+
 sub install_mingw_make {
 	my $self = shift;
 
-	my $filelist = $self->install_binary(
-		name => 'mingw-make',
-		url  => $self->_binary_url('mingw-make'),
-	);
+	my $filelist;
+
+	if ( 4 == $self->gcc_version() ) {
+		$filelist = $self->install_binary(
+			name    => 'mingw-make',
+			url     => $self->_binary_url('mingw-make'),
+			license => {
+				'doc/COPYING' => 'gmake/COPYING',
+				'doc/AUTHORS' => 'gmake/AUTHORS',
+			},
+			install_to => { 'bin/gmake.exe' => 'c/bin/gmake.exe', },
+		);
+	} else {
+		$filelist = $self->install_binary(
+			name    => 'mingw-make',
+			url     => $self->_binary_url('mingw-make'),
+			license => {
+				'doc/mingw32-make/COPYING'      => 'gmake/COPYING',
+				'doc/mingw32-make/README.mingw' => 'gmake/README.mingw.txt',
+			},
+			install_to =>
+			  { 'bin/mingw32-make.exe' => 'c/bin/mingw32-make.exe', },
+		);
+	} ## end else [ if ( 4 == $self->gcc_version...)]
 
 	$self->insert_fragment( 'mingw_make', $filelist );
 
@@ -285,147 +347,6 @@ sub install_mingw_make {
 
 __END__
 
-# =pod
-
-# =head2 install_gcc
-
-  # $dist->install_gcc
-
-# The C<install_gcc> method installs the B<GNU C Compiler> into the
-# distribution, and is typically installed during "C toolchain" build
-# phase.
-
-# It provides the appropriate arguments to several C<install_binary>
-# calls. The default C<install_gcc> method installs two binary
-# packages, the core compiler 'gcc-core' and the C++ compiler 'gcc-c++'.
-
-# Returns true or throws an exception on error.
-
-# =cut
-
-sub install_gcc {
-	my $self = shift;
-
-	# Install the compilers (gcc)
-	my $fl = $self->install_binary(
-		name    => 'gcc-core',
-		url     => $self->_binary_url('gcc-core'),
-		license => {
-			'COPYING'     => 'gcc/COPYING',
-			'COPYING.lib' => 'gcc/COPYING.lib',
-			'doc/mingw-runtime/Contributors' => 'mingw/Contributors',
-			'doc/mingw-runtime/Disclaimer'   => 'mingw/Disclaimer',
-		},
-	);
-
-	$self->insert_fragment( 'gcc_core', $fl );
-
-	$fl = $self->install_binary( 
-		name => 'gcc-g++', 
-		url  => $self->_binary_url('gcc-g++'),
-	);
-
-	$self->insert_fragment( 'gcc_gplusplus', $fl );
-
-	return 1;
-} ## end sub install_gcc
-
-# =pod
-
-# =head2 install_binutils
-
-  # $dist->install_binutils
-
-# The C<install_binutils> method installs the C<GNU binutils> package into
-# the distribution.
-
-# The most important of these is C<dlltool.exe>, which is used to extract
-# static library files from .dll files. This is needed by some libraries
-# to let the Perl interfaces build against them correctly.
-
-# Returns true or throws an exception on error.
-
-# =cut
-
-sub install_binutils {
-	my $self = shift;
-
-	my $filelist = $self->install_binary(
-		name    => 'binutils',
-		url     => $self->_binary_url('binutils'),
-		license => {
-			'Copying'     => 'binutils/Copying',
-			'Copying.lib' => 'binutils/Copying.lib',
-		},
-	);
-	$self->bin_dlltool() =
-	  $self->_file( 'c', 'bin', 'dlltool.exe' );
-	unless ( -x $self->bin_dlltool ) {
-		PDWiX->throw(q{Can't execute dlltool});
-	}
-
-	$self->insert_fragment( 'binutils', $filelist );
-
-	return 1;
-} ## end sub install_binutils
-
-# =pod
-
-# =head2 install_mingw_runtime
-
-  # $dist->install_mingw_runtime
-
-# The C<install_mingw_runtime> method installs the MinGW runtime package
-# into the distribution, which is basically the MinGW version of libc and
-# some other very low level libs.
-
-# Returns true or throws an exception on error.
-
-# =cut
-
-sub install_mingw_runtime {
-	my $self = shift;
-
-	my $filelist = $self->install_binary(
-		name    => 'mingw-runtime',
-		url     => $self->_binary_url('mingw-runtime'),
-		license => {
-			'doc/mingw-runtime/Contributors' => 'mingw/Contributors',
-			'doc/mingw-runtime/Disclaimer'   => 'mingw/Disclaimer',
-		},
-	);
-
-	$self->insert_fragment( 'mingw_runtime', $filelist );
-
-	return 1;
-} ## end sub install_mingw_runtime
-
-# =pod
-
-# =head2 install_win32api
-
-  # $dist->install_win32api
-
-# The C<install_win32api> method installs C<MinGW win32api> layer, to
-# allow C code to compile against native Win32 APIs.
-
-# Returns true or throws an exception on error.
-
-# =cut
-
-sub install_win32api {
-	my $self = shift;
-
-	my $filelist = $self->install_binary( 
-		name => 'w32api',
-		url  => $self->_binary_url('w32api'),
-	);
-
-	$self->insert_fragment( 'w32api', $filelist );
-
-	return 1;
-}
-
 =pod
 
 =head1 DIAGNOSTICS
@@ -450,7 +371,7 @@ Curtis Jewell E<lt>csjewell@cpan.orgE<gt>
 
 =head1 SEE ALSO
 
-L<Perl::Dist|Perl::Dist>, L<Perl::Dist::WiX|Perl::Dist::WiX>, 
+L<Perl::Dist::WiX|Perl::Dist::WiX>, 
 L<http://ali.as/>, L<http://csjewell.comyr.com/perl/>
 
 =head1 COPYRIGHT AND LICENSE
