@@ -8,7 +8,7 @@ Perl::Dist::WiX::Tag::Directory - <Directory> tag that knows how to search its c
 
 =head1 VERSION
 
-This document describes Perl::Dist::WiX::Tag::Directory version 1.200.
+This document describes Perl::Dist::WiX::Tag::Directory version 1.200002.
 
 =head1 SYNOPSIS
 
@@ -51,7 +51,7 @@ use Params::Util qw( _STRING );
 use Digest::CRC qw( crc32_base64 );
 require Perl::Dist::WiX::Exceptions;
 
-our $VERSION = '1.200';
+our $VERSION = '1.200002';
 $VERSION =~ s/_//ms;
 
 extends 'WiX3::XML::Directory';
@@ -207,8 +207,8 @@ sub search_dir {
 	return undef unless defined $path;
 
 	$self->trace_line( 3, "Looking for $path_to_find\n" );
-	$self->trace_line( 4, "  in:      $path.\n" );
-	$self->trace_line( 5, "  descend: $descend exact: $exact.\n" );
+	$self->trace_line( 4, "   in:      $path.\n" );
+	$self->trace_line( 5, "   descend: $descend exact: $exact.\n" );
 
 	# If we're at the correct path, exit with success!
 	if ( ( defined $path ) && ( $path_to_find eq $path ) ) {
@@ -232,6 +232,8 @@ sub search_dir {
 		$self->trace_line( 4, "Not a subset in: $path.\n" );
 		$self->trace_line( 5, "  To find: $path_to_find.\n" );
 		return undef;
+	} else {
+		$self->trace_line( 4, "Subset of $path_to_find in $path.\n" );
 	}
 
 	# Check each of our branches.
@@ -251,7 +253,14 @@ sub search_dir {
 	}
 
 	# If we get here, we did not find a lower directory.
-	return $exact ? undef : $self;
+	$self->trace_line( 4, "Did not find lower directory than $path.\n" );
+	if ($exact) {
+		$self->trace_line( 5, "Returning undef.\n" );
+		return undef;
+	} else {
+		$self->trace_line( 5, "Returning object for $path.\n" );
+		return $self;
+	}
 } ## end sub search_dir
 
 sub _add_directory_recursive {
@@ -266,7 +275,8 @@ sub _add_directory_recursive {
 		return undef;
 	}
 
-	my $directory = $self->search_dir(
+	my $path_to_add = $path_to_find . q{\\} . $dir_to_add;
+	my $directory   = $self->search_dir(
 		path_to_find => $path_to_find,
 		descend      => 1,
 		exact        => 1,
@@ -275,7 +285,8 @@ sub _add_directory_recursive {
 	if ( defined $directory ) {
 		return $directory->add_directory(
 			name => $dir_to_add,
-			id   => crc32_base64( $path_to_find . $dir_to_add ),
+			id   => crc32_base64($path_to_add),
+			path => $path_to_add,
 		);
 	} else {
 		my ( $volume, $dirs, undef ) = splitpath( $path_to_find, 1 );
@@ -293,7 +304,8 @@ sub _add_directory_recursive {
 		}
 		return $dir->add_directory(
 			name => $dir_to_add,
-			id   => crc32_base64( $path_to_find . $dir_to_add ),
+			id   => crc32_base64($path_to_add),
+			path => $path_to_add,
 		);
 
 	} ## end else [ if ( defined $directory)]
